import { async } from 'most-subject';
import { Stream } from 'most';

import { emitter } from './state/portal';

export var netStatuses = {
  halt: 'halt',
  load: 'load',
  guest: 'guest',
  active: 'active'
};

var statusList = ['halt', 'load', 'guest', 'active'];

var iso = {
  from(status) {
    var index = statusList.indexOf(status);
    if (index === -1) return 0;
    return index;
  },
  to(index) {
    if (index < 0 || index > statusList.length - 1) return Status.empty();
    return statusList[index];
  },
  pro(fn) {
    return status => iso.to(fn(iso.from(status)));
  }
};

var Status = {
  empty: () => netStatuses.halt,
  top: () => netStatuses.active,
  next: iso.pro(n => n + 1),
  back: iso.pro(n => n - 1),
  eq: (s1, s2) => s1 === s2,
  gt(s1, s2) {
    return iso.from(s1) > iso.from(s2);
  },
  gte: (s1, s2) => Status.eq(s1, s2) || Status.gt(s1, s2),
  max: (s1, s2) => Status.gt(s1, s2) ? s1 : s2,
  min: (s1, s2) => Status.gt(s1, s2) ? s2 : s1,
  ensure(obj) {
    switch (obj) {
      case 'halt':
        return 'halt';
      case 'load':
        return 'load';
      case 'guest':
        return 'guest';
      case 'active':
        return 'active';
      default:
        return Status.empty();
    }
  },
  is: isStatus
};

function isStatus(obj) {
  return netStatuses.halt === obj || netStatuses.load === obj || netStatuses.guest === obj || netStatuses.active === obj;
}

/*::
(async: AsyncSubj)
*/

function pushList(list, subj) {
  for (var i = 0, ln = list.length; i < ln; i++) {
    subj.next(list[i]);
  }
}

export function netStatusGuard(minStatus, guard, source) {
  var result = async();
  var buffer = [];
  var active = false;
  emitter.on('cleanup', () => {
    buffer = [];
    // active = false
  });
  source.subscribe({
    next(x) {
      active ? result.next(x) : buffer.push(x);
    },
    error(x) {
      result.error(x);
    },
    complete(x) {
      pushList(buffer, result);
      result.complete(x);
    }
  });

  guard.map(x => Status.gte(x, minStatus)).skipRepeats().observe(x => {
    if (x) pushList(buffer, result);
    active = x;
    buffer = [];
  });
  return result;
}

export default Status;
//# sourceMappingURL=net-status.js.map