function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

import { trim, fromPairs } from 'ramda';
import 'mtproto-shared';

import dcStoreKeys from '../../util/dc-store-keys';

import { dispatch } from '../../state';
import random from '../secure-random';
import { MAIN } from '../../state/action';
import { bytesFromHex } from '../../bin';
import Thread from '../networker';

import { toDCInt } from '../../state/carrier';

import { toDCNumber } from '../../newtype.h';

export default (() => {
  var _ref = _asyncToGenerator(function* (storage, dcMap, uid) {
    var iAuth = {};
    var iSalt = {};
    var iHome = 2;
    var flags = {
      networker: false,
      auth: false,
      homeDC: false,
      net: false
    };
    var networker = {
      ids: []
    };
    var authList = {
      authKey: [],
      salt: [],
      session: []
    };
    for (var _iterator = dcMap.keys(), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref2;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref2 = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref2 = _i.value;
      }

      var _dc = _ref2;

      // flags = { ...flags, networker: true }
      var dcInt = toDCInt(_dc);
      var inactive = true;
      var fields = {
        authKey: [],
        salt: [],
        session: []
      };
      var d = dcStoreKeys(_dc);
      var salt = yield storage.get(d.saltKey);
      // const salt = checkString(saltRaw)
      if (Array.isArray(salt)) {
        iSalt = Object.assign({}, iSalt, { [_dc | 0]: salt });

        inactive = false;
        flags = Object.assign({}, flags, {
          networker: true,
          auth: true
        });
        fields = Object.assign({}, fields, {
          salt
        });
        authList = Object.assign({}, authList, {
          salt: [...authList.salt, dcInt]
          // dispatch(AUTH.SET_SERVER_SALT(salt, dc))
        });
      }
      var auth = yield storage.get(d.authKey);
      if (Array.isArray(auth)) {
        iAuth = Object.assign({}, iAuth, { [_dc | 0]: auth });

        var saltKey = salt === false ? bytesFromHex('AAAAAAAAAAAAAAAA') : salt;

        iSalt = Object.assign({}, iSalt, { [_dc | 0]: saltKey });

        inactive = false;
        flags = Object.assign({}, flags, {
          networker: true,
          auth: true
        });
        new Thread(_dc, auth, saltKey, uid);
        fields = Object.assign({}, fields, {
          salt: saltKey,
          authKey: auth
        });
        authList = Object.assign({}, authList, {
          salt: [...new Set([...authList.salt, dcInt])],
          authKey: [...authList.authKey, dcInt]
          // dispatch(AUTH.SET_AUTH_KEY(auth, dc))
        });
      }
      if (!inactive) networker = Object.assign({}, networker, {
        ids: [...networker.ids, dcInt],
        [_dc | 0]: fields
      });
    }
    var nearestRaw = yield storage.get('nearest_dc');
    var nearest = checkNumber(nearestRaw);
    var act = {
      flags,
      networker,
      auth: authList
      /*:: homeDC: toDCInt(0), */
    };
    if (nearest !== false) {
      iHome = nearest;
      flags = Object.assign({}, flags, { homeDC: true });
      var dcIntNearest = toDCInt(nearest + 0);
      act = Object.assign({}, act, {
        flags,
        homeDC: dcIntNearest
        // dispatch(MAIN.DC_DETECTED(nearest))
      });
    } //else
    // dispatch(MAIN.MODULE_LOADED())
    // dispatch(MAIN.DC_DETECTED(2))
    // dispatch(makeCarrierAction(act))
    var session = createSessions(getDcList(iAuth, iSalt, iHome));
    var finalAction = {
      auth: iAuth,
      salt: iSalt,
      home: iHome,
      uid,
      session
    };
    dispatch(MAIN.STORAGE_IMPORTED(finalAction), uid);
  });

  return function loadStorage(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

function getDcList(auth, salt, home) {
  var dcList = [].concat(Object.keys(auth), Object.keys(salt), [home]).filter(isFinite).map(e => parseInt(e, 10));
  return [...new Set(dcList)];
}

//$off
var createSessions = dcList => fromPairs(dcList.map(dc => [dc, random(new Array(8))]));

function checkString(source) {
  if (source == null || typeof source !== 'string') return false;
  var trimmed = trim(source);
  if (trimmed.length === 0) return false;
  return bytesFromHex(trimmed);
}

function checkNumber(source) {
  if (source == null || typeof source !== 'number') return false;
  if (!isFinite(source)) return false;
  return (/*:: toDCNumber(*/source
  ); /*::)*/
}
//# sourceMappingURL=load-storage.js.map