import 'redux';
import { of } from 'most';
import { replace, toPairs, chain, o, contains, trim, pipe, identity } from 'ramda';
import Logger from 'mtproto-logger';
var log = Logger`redux-core`;

import { of as ofRight } from 'apropos';

import './index.h';
// import warning from 'Util/warning'
import { subject } from '../property';

export var tryAddUid = () => next => action => {
  if (typeof action !== 'object' || action == null) return;
  if (typeof action.uid === 'string') return (() => next(action))();
  if (typeof action.payload === 'object' && action.payload != null && typeof action.payload.uid === 'string') action.uid = action.payload.uid;
  return next(action);
};

export var skipEmptyMiddleware = () => next => action => {
  if (getActionType(action) === 'networker/sent delete' && Array.isArray(action.payload) && action.payload.length === 0) //TODO Remove hardcode
    log`skip empty`(action.type);else return next(action);
};

export var debounceMiddleware = ({ dispatch }) => next => {
  var actionStream = subject({ type: '', payload: {}, meta: { debounced: false } });
  actionStream.tap(log`action`).map(action => {
    action.meta.debounced = true;
    return action;
  }).concatMap(val => of(val).delay(50)).observe(dispatch);
  return action => {
    if (action.meta && action.meta.debounced) return next(action);
    return actionStream.next(action);
  };
};

/*::
declare var actionObject: ActionObject
declare var dataArray: Array<any>
const actionArray = { ...actionObject, payload: dataArray }

type ActionArray = typeof actionArray
*/

// type BatchConfig = {[scope: string]: string[]}

var removeTypeIndex = replace(/\[\d+\] /, '');

var getActionType = pipe(e => e.type || '', removeTypeIndex, trim);

// const actionPairs = ([scope, acts]: [string, string[]]): string[] => acts.map(
//   (name: string): string => [scope, name].join('/'))

// const configToActionTypes: (config: BatchConfig) => string[] =
//   //$ FlowIssue
//   o(chain(actionPairs), toPairs)
//
// function ensurePushMap(
//   actionType: string,
//   value: ActionArray,
//   map: Map<string, ActionArray>
// ) {
//   const stored = map.get(actionType)
//   if (stored !== undefined) {
//     stored.payload = stored.payload.concat(value.payload)
//   } else {
//     value.meta.debounced = true
//     map.set(actionType, value)
//   }
// }

// const BATCH_TIMEOUT = 100

// export function batch(config: BatchConfig) {
//   const actions = configToActionTypes(config)
//   const isBatch = (model: string): boolean => contains(model, actions)
//   return ({ dispatch }: *) => (next: *) => {
//     let actionMap: Map<string, ActionArray> | null = null
//     function dispatchAll() {
//       if (actionMap == null) return
//       for (const action of actionMap.values())
//         dispatch(action)
//       actionMap = null
//     }
//     return (action: ActionObject) => {
//       const actionType = getActionType(action)
//       if (isBatch(actionType) && !action.meta.debounced) {
//         if (actionMap == null) {
//           actionMap = new Map
//           setTimeout(dispatchAll, BATCH_TIMEOUT)
//         }
//         if (Array.isArray(action.payload)) {
//           /*::
//           (action: ActionArray)
//           */
//           ensurePushMap(actionType, action, actionMap)
//         }
//       } else {
//         return next(action)
//       }
//     }
//   }
// }


export var normalizeActions = defaults => () => next => action => next(Object.assign({}, defaults, action));
//# sourceMappingURL=middleware.js.map