'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeActions = exports.debounceMiddleware = exports.skipEmptyMiddleware = exports.tryAddUid = undefined;

require('redux');

var _most = require('most');

var _ramda = require('ramda');

var _mtprotoLogger = require('mtproto-logger');

var _mtprotoLogger2 = _interopRequireDefault(_mtprotoLogger);

var _apropos = require('apropos');

require('./index.h');

var _property = require('../property');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var log = _mtprotoLogger2.default`redux-core`;
// import warning from 'Util/warning'
var tryAddUid = exports.tryAddUid = () => next => action => {
  if (typeof action !== 'object' || action == null) return;
  if (typeof action.uid === 'string') return (() => next(action))();
  if (typeof action.payload === 'object' && action.payload != null && typeof action.payload.uid === 'string') action.uid = action.payload.uid;
  return next(action);
};

var skipEmptyMiddleware = exports.skipEmptyMiddleware = () => next => action => {
  if (getActionType(action) === 'networker/sent delete' && Array.isArray(action.payload) && action.payload.length === 0) //TODO Remove hardcode
    log`skip empty`(action.type);else return next(action);
};

var debounceMiddleware = exports.debounceMiddleware = ({ dispatch }) => next => {
  var actionStream = (0, _property.subject)({ type: '', payload: {}, meta: { debounced: false } });
  actionStream.tap(log`action`).map(action => {
    action.meta.debounced = true;
    return action;
  }).concatMap(val => (0, _most.of)(val).delay(50)).observe(dispatch);
  return action => {
    if (action.meta && action.meta.debounced) return next(action);
    return actionStream.next(action);
  };
};

/*::
declare var actionObject: ActionObject
declare var dataArray: Array<any>
const actionArray = { ...actionObject, payload: dataArray }

type ActionArray = typeof actionArray
*/

// type BatchConfig = {[scope: string]: string[]}

var removeTypeIndex = (0, _ramda.replace)(/\[\d+\] /, '');

var getActionType = (0, _ramda.pipe)(e => e.type || '', removeTypeIndex, _ramda.trim);

// const actionPairs = ([scope, acts]: [string, string[]]): string[] => acts.map(
//   (name: string): string => [scope, name].join('/'))

// const configToActionTypes: (config: BatchConfig) => string[] =
//   //$ FlowIssue
//   o(chain(actionPairs), toPairs)
//
// function ensurePushMap(
//   actionType: string,
//   value: ActionArray,
//   map: Map<string, ActionArray>
// ) {
//   const stored = map.get(actionType)
//   if (stored !== undefined) {
//     stored.payload = stored.payload.concat(value.payload)
//   } else {
//     value.meta.debounced = true
//     map.set(actionType, value)
//   }
// }

// const BATCH_TIMEOUT = 100

// export function batch(config: BatchConfig) {
//   const actions = configToActionTypes(config)
//   const isBatch = (model: string): boolean => contains(model, actions)
//   return ({ dispatch }: *) => (next: *) => {
//     let actionMap: Map<string, ActionArray> | null = null
//     function dispatchAll() {
//       if (actionMap == null) return
//       for (const action of actionMap.values())
//         dispatch(action)
//       actionMap = null
//     }
//     return (action: ActionObject) => {
//       const actionType = getActionType(action)
//       if (isBatch(actionType) && !action.meta.debounced) {
//         if (actionMap == null) {
//           actionMap = new Map
//           setTimeout(dispatchAll, BATCH_TIMEOUT)
//         }
//         if (Array.isArray(action.payload)) {
//           /*::
//           (action: ActionArray)
//           */
//           ensurePushMap(actionType, action, actionMap)
//         }
//       } else {
//         return next(action)
//       }
//     }
//   }
// }


var normalizeActions = exports.normalizeActions = defaults => () => next => action => next(Object.assign({}, defaults, action));
//# sourceMappingURL=middleware.js.map