'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.statuses = undefined;
exports.statusGuard = statusGuard;

var _mostSubject = require('most-subject');

var _most = require('most');

var _portal = require('./state/portal');

/*::
opaque type EmptyStatus: ModuleStatus = 'init'
*/

var statuses = exports.statuses = {
  init: 'init',
  importStorage: 'import storage',
  loaded: 'loaded',
  dcDetected: 'dc detected',
  activated: 'activated'
};

var statusList = ['init', 'import storage', 'loaded', 'dc detected', 'activated'];

var iso = {
  from(status) {
    var index = statusList.indexOf(status);
    if (index === -1) return 0;
    return index;
  },
  to(index) {
    if (index < 0 || index > statusList.length - 1) return Status.empty();
    return statusList[index];
  },
  pro(fn) {
    return status => iso.to(fn(iso.from(status)));
  }
};

var Status = {
  empty: () => statuses.init,
  next: iso.pro(n => n + 1),
  back: iso.pro(n => n - 1),
  eq: (s1, s2) => s1 === s2,
  gt(s1, s2) {
    return iso.from(s1) > iso.from(s2);
  },
  gte: (s1, s2) => Status.eq(s1, s2) || Status.gt(s1, s2),
  max: (s1, s2) => Status.gt(s1, s2) ? s1 : s2,
  ensure(obj) {
    switch (obj) {
      case 'init':
        return 'init';
      case 'import storage':
        return 'import storage';
      case 'loaded':
        return 'loaded';
      case 'dc detected':
        return 'dc detected';
      case 'activated':
        return 'activated';
      default:
        return Status.empty();
    }
  },
  is: isStatus
};

function isStatus(obj) {
  return statuses.init === obj || statuses.importStorage === obj || statuses.loaded === obj || statuses.dcDetected === obj || statuses.activated === obj;
}

exports.default = Status;


/*::
(async: AsyncSubj)
*/

function pushList(list, subj) {
  for (var i = 0, ln = list.length; i < ln; i++) {
    subj.next(list[i]);
  }
}

function statusGuard(minStatus, guard, source) {
  var result = (0, _mostSubject.async)();
  var buffer = [];
  var active = false;
  _portal.emitter.on('cleanup', () => {
    buffer = [];
    // active = false
  });
  source.subscribe({
    next(x) {
      active ? result.next(x) : buffer.push(x);
    },
    error(x) {
      result.error(x);
    },
    complete(x) {
      pushList(buffer, result);
      result.complete(x);
    }
  });

  guard.map(x => Status.gte(x, minStatus)).skipRepeats().observe(x => {
    if (x) pushList(buffer, result);
    active = x;
    buffer = [];
  });
  return result;
}
//# sourceMappingURL=status.js.map